local DASH_THRESHOLD = 2
local DASH_MAX = DASH_THRESHOLD + 3
local NORMALSPEED = 36*FRACUNIT

local isMetal = function(skin)
	return skin == "running_msonic"
end

//Code for Metal's fume and dashmode visual effects used from SMS Alfredo's SRB2 Chars 

//Followmobj
freeslot("MT_RCHAROVERLAY", "S_RMETALJETFUMEFLASH", "S_RMETALJETFUME1")

mobjinfo[MT_RCHAROVERLAY] = {
	doomednum = -1,
	spawnstate = S_INVISIBLE,
	seestate = S_RMETALJETFUMEFLASH,
	raisestate = S_RMETALJETFUME1,
	spawnhealth = 1000,
	reactiontime = 8,
	speed = 8,
	mass = 16,
	radius = 8*FRACUNIT,
	height = 16*FRACUNIT,
	flags = MF_NOBLOCKMAP|MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOGRAVITY|MF_SCENERY|MF_DONTENCOREMAP,
}

//Metal Sonic's fume
states[S_RMETALJETFUMEFLASH] = {SPR_PLAY, N|FF_ANIMATE|FF_FULLBRIGHT, 2, nil, 1, 1, S_RMETALJETFUME1}
states[S_RMETALJETFUME1] = {SPR_PLAY, K|FF_ANIMATE|FF_FULLBRIGHT, -1, nil, 2, 1, S_NULL}

local P_DoRMetalJetFume = function(player, fume)
	local FUME_SKINCOLORS =
	{
		SKINCOLOR_CYAN,			//SKINCOLOR_ICY
	}
	local mo = player.mo
	local angle = player.frameangle
	local dist
	local dashmode = player.dashmode
	local underwater = mo.eflags & MFE_UNDERWATER
	local stat = fume.state

	if (player.speed == 0 or player.kartstuff[k_drift] > 0 or player.kartstuff[k_drift] < 0 or not P_IsObjectOnGround(player.mo) or player.kartstuff[k_spinouttimer] > 0 or player.kartstuff[k_boostcharge] > 0) // turn invisible when not in a coherent movement state
		fume.state = fume.info.spawnstate
		fume.color = FUME_SKINCOLORS[1]
		return
	end

	if (underwater) // No fume underwater spawn bubbles instead!
		fume.movedir	= $ + FixedAngle(FixedDiv(2 * player.speed, 3 * mo.scale))
		fume.movefactor = $ + player.speed
		if (fume.movefactor > FixedDiv(2 * player.normalspeed, 3 * mo.scale))
			local radiusV = 4*FRACUNIT
			local radiusX = P_ReturnThrustX(mo, angle, -mo.radius)
			local radiusY = P_ReturnThrustY(mo, angle, -mo.radius)
			if player.speed > 0
				radiusX = $*2/3
				radiusY = $*2/3
			end
			local factorX = P_ReturnThrustX(mo, angle + ANGLE_90, mo.scale)
			local factorY = P_ReturnThrustY(mo, angle + ANGLE_90, mo.scale)
			local i = -1 while i < 2
				local offsetH = i*P_ReturnThrustX(fume, fume.movedir, radiusV)
				local offsetV = i*P_ReturnThrustY(fume, fume.movedir, radiusV)
				local x = mo.x + radiusX + FixedMul(offsetH, factorX)
				local y = mo.y + radiusY + FixedMul(offsetH, factorY)
				local z = mo.z + (mo.height >> 1) + offsetV
				P_SpawnMobj(x, y, z, MT_SMALLBUBBLE)//.scale = mo.scale >> 1
				i = $ + 2
			end

			fume.movefactor = 0
		end
		if (player.speed > 0)
			if (stat != fume.info.spawnstate)
				fume.threshold = 0
				fume.state = fume.info.spawnstate
			end
			return
		end
	end

	if (stat == fume.info.spawnstate and not underwater) // If currently inivisble, activate!
		stat = fume.info.seestate
		fume.state = stat
		P_SetScale(fume, mo.scale)
	end
	K_MatchGenericExtraFlags(fume, player.mo)

	if (dashmode > DASH_THRESHOLD and stat != fume.info.seestate and not underwater) // If in dashmode, grow really big and flash
		fume.destscale = mo.scale
		if leveltime%2
			fume.flags2 = $ | MF2_DONTDRAW
		else
			fume.flags2 = $ &~ MF2_DONTDRAW
		end
	else // Otherwise, pick a size depending on speed and proximity to dashmode
		if (dashmode == DASH_THRESHOLD and dashmode > fume.movecount and not underwater) // If just about to enter dashmode, play the startup animation again
			stat = fume.info.seestate
			fume.state = stat
			P_SetScale(fume, mo.scale << 1)
		end
		fume.destscale = (mo.scale + FixedDiv(player.speed, player.normalspeed)) / (underwater and 6 or 3)
		

		if (underwater)
			fume.frame = (fume.frame & FF_FRAMEMASK) | FF_ANIMATE | (P_RandomRange(0, 9) * FF_TRANS10)
			fume.threshold = 1
		elseif (fume.threshold)
			fume.frame = (fume.frame & FF_FRAMEMASK) | (states[fume.state].frame &~ FF_FRAMEMASK)
			fume.threshold = 0
		end
	end

	fume.movecount = dashmode // keeps track of previous dashmode value so we know whether Metal is entering or leaving it
	fume.eflags = (fume.flags2 & ~MF2_OBJECTFLIP) | (mo.flags2 & MF2_OBJECTFLIP) // Make sure to flip in reverse gravity!
	fume.eflags = (fume.eflags & ~MFE_VERTICALFLIP) | (mo.eflags & MFE_VERTICALFLIP) // Make sure to flip in reverse gravity!
	fume.colorized = true

	// Finally, set its position
	dist = -mo.radius - FixedMul(fume.info.radius, fume.destscale - mo.scale/3)

	P_TeleportMove(fume,
	mo.x + P_ReturnThrustX(fume, angle, dist),
	mo.y + P_ReturnThrustY(fume, angle, dist),
	mo.z + ((mo.height - fume.height) >> 1))

	// If dashmode is high enough, spawn a trail
	if (player.speed > 53*player.mo.scale and P_IsObjectOnGround(player.mo) and not underwater)
		P_SpawnGhostMobj(fume)
	end
end

//Reset the variables
//Using a custom spinning variable since Kart uses PF_SPINNING for other purposes
addHook("PlayerSpawn", function(player)
	player.spinning = false
	player.srb2hit = false
	player.rcharfollowmobj = nil
	player.dumbmomx = nil
	player.dumbmomy = nil
	player.dashmode = 0
	player.normalspeed = NORMALSPEED
end)



// Movement code
local SRB2Movement = function(player)

	//Dashmode garbage
	if player.mo.skin == "running_msonic"
		local prevdashmode = player.dashmode
		local topspeed = player.speed >= 42*player.mo.scale

		if ((topspeed) or (player.pflags & PF_STARTDASH))
			if (player.dashmode < DASH_MAX)
				player.dashmode = $+1 // Counter. Adds 1 to dash mode per tic in top speed.
			end
			
			if (player.dashmode == DASH_THRESHOLD) // This isn't in the ">=" equation because it'd cause the sound to play infinitely.
				S_StartSound(player.mo, sfx_kc4d) // If the player enters dashmode, play this sound on the the tic it starts.
			end
		elseif (not topspeed) and not player.spinning
		and not (player.pflags&PF_SLIDING) and not player.kartstuff[k_pogospring]
			if (player.dashmode > 1)
				player.dashmode = $-1 // Rather than lose it all, it gently counts back down!
				if ((player.dashmode+3) >= DASH_THRESHOLD and player.dashmode < DASH_THRESHOLD)
					// S_StartSound(player.mo, sfx_kc65) Remove the // if you want the dashmode power off sound. Removed cause its kinda annoying during races.
					player.mo.colorized = false
					player.mo.color = player.skincolor
				end
			else
				player.dashmode = 0
			end
		end

		if (player.speed > 53*player.mo.scale and P_IsObjectOnGround(player.mo))
			local ghost = P_SpawnGhostMobj(player.mo) // Spawns afterimages
			ghost.fuse = 2 // Makes the images fade quickly
			if (leveltime/2) & 1
				ghost.colorized = false
				ghost.color = player.skincolor
			end
		end
	end
end

addHook("ThinkFrame", do
for player in players.iterate
	if player.mo and player.mo.valid and isMetal(player.mo.skin)
	and not (player.kartstuff[k_comebacktimer] <= 0
	and player.kartstuff[k_bumper] <= 0 and G_BattleGametype())
		player.mo.radius = 23*player.mo.scale/2
		player.kartstuff[k_enginesnd] = 0
		player.pflags = $&~PF_SKIDDOWN

		
		//Determine if we're Metal Sonic
		local runningmetal = player.mo.skin == "running_msonic"
			
		//Dying animation
		if not player.health
			player.mo.momz = $-(P_GetMobjGravity(player.mo)*2/3)
			
			if runningmetal and not (player.mo.fuse % 8)
				local r = player.mo.radius >> FRACBITS
				local explosion = P_SpawnMobj(
					player.mo.x + (P_RandomRange(r, -r) << FRACBITS),
					player.mo.y + (P_RandomRange(r, -r) << FRACBITS),
					player.mo.z + (P_RandomKey(player.mo.height >> FRACBITS) << FRACBITS),
					MT_SONIC3KBOSSEXPLODE)
				S_StartSound(explosion, sfx_s3kb4)
			end
		//Metal Sonic's "animations"
		elseif runningmetal and not (player.kartstuff[k_drift] > 0 or player.kartstuff[k_drift] < 0)
			if player.dashmode >= DASH_THRESHOLD
				player.mo.frame = R
			elseif player.speed > 21*player.mo.scale
				player.mo.frame = Q
			else
				player.mo.frame = P
			end
		elseif runningmetal
			if (player.kartstuff[k_drift] > 0 or player.kartstuff[k_drift] < 0)
				player.mo.frame = Z+11
			end
		end
		//
		//Prevent the rest of the script under certain circumstances
		//
		if player.kartstuff[k_spinouttimer] > 0
		or player.pflags&(PF_SLIDING|PF_MACESPIN|PF_ITEMHANG|PF_CARRIED|PF_ROPEHANG)
		or (player.mo.tracer and player.mo.tracer.valid and player.mo.tracer.type == MT_TUBEWAYPOINT)
		or not player.health or player.pflags&PF_STASIS
		or player.kartstuff[k_respawn] or player.kartstuff[k_squishedtimer]
		or player.exiting or player.mo.eflags&MFE_SPRUNG
			player.spinning = false
			player.dumbmomx = nil
			player.dumbmomy = nil
			
			if player.srb2hit and player.kartstuff[k_spinouttimer]
				if player.srb2hitter
					player.mo.z = $+4*player.mo.scale
					player.mo.momz = 6*player.mo.scale
					if player.srb2hitter == -1
					or not (player.srb2hitter and player.srb2hitter.valid)
						P_InstaThrust(player.mo, player.mo.angle, -10*player.mo.scale)
					else
						P_InstaThrust(player.mo,
						R_PointToAngle2(player.mo.x, player.mo.y,
						player.srb2hitter.x, player.srb2hitter.y), -10*player.mo.scale)
					end
					
				end
				player.frameangle = R_PointToAngle2(player.rmomx, player.rmomy, 0,0)
				player.mo.momz = $-(P_GetMobjGravity(player.mo)*3/10)
				if P_IsObjectOnGround(player.mo)
					player.kartstuff[k_spinouttimer] = 0
				end
			else
				player.srb2hit = false
			end
			player.srb2hitter = nil
			
			//Followmobj
			if (runningmetal) and player.mo.health
				if player.rcharfollowmobj and player.rcharfollowmobj.valid
					if player.rcharfollowmobj.type != MT_RCHAROVERLAY
						player.rcharfollowmobj = nil
					elseif player.rcharfollowmobj.target != player.mo
						P_KillMobj(player.rcharfollowmobj)
						player.rcharfollowmobj = nil
					else
						P_DoRMetalJetFume(player, player.rcharfollowmobj)
					end
				else
					player.rcharfollowmobj = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z, MT_RCHAROVERLAY)
					player.rcharfollowmobj.target = player.mo
				end
			end
			continue
		end
		player.srb2hit = false
		player.srb2hitter = nil
		
		//Followmobj
		if (runningmetal) and player.mo.health
			if player.rcharfollowmobj and player.rcharfollowmobj.valid
				if player.rcharfollowmobj.type != MT_RCHAROVERLAY
					player.rcharfollowmobj = nil
				elseif player.rcharfollowmobj.target != player.mo
					P_KillMobj(player.rcharfollowmobj)
					player.rcharfollowmobj = nil
				else
					P_DoRMetalJetFume(player, player.rcharfollowmobj)
				end
			else
				player.rcharfollowmobj = P_SpawnMobj(player.mo.x, player.mo.y, player.mo.z, MT_RCHAROVERLAY)
				player.rcharfollowmobj.target = player.mo
			end
		end
		SRB2Movement(player)
	end
end end)