addHook("ThinkFrame", do
	for player in players.iterate
		if player.mo and player.mo.skin == "running_tails"
					and P_IsObjectOnGround(player.mo)
		if player.sprung == nil
			player.sprung = false
		end
					player.sprung = false
					end
				end
			end)

addHook("MobjMoveCollide", function(mobj, spring)
	if (spring.flags & MF_SPRING) 
		if (mobj.z <= spring.z + spring.height)
		and (spring.z <= mobj.z + mobj.height)
		and (mobj.skin == "running_tails")
		and not mobj.player.powers[pw_super]
			mobj.player.sprung = true
		end
	end
	if (spring.type == MT_SPRINGSHELL and spring.health > 0) then
		local tmz = (spring.eflags & MFE_VERTICALFLIP and -((mobj.z + mobj.height) or 1) or mobj.z)
		local tmznext = (spring.eflags & MFE_VERTICALFLIP and -mobj.momz or mobj.momz) + tmz
		local thzh = (spring.eflags & MFE_VERTICALFLIP and -(spring.z or 1) or spring.z + spring.height)
		local sprarea = FixedMul(8*FRACUNIT, spring.scale) * P_MobjFlip(spring)
		if (((tmznext <= thzh) and (tmz > thzh)) or ((tmznext > thzh - sprarea) and (tmznext < thzh))) then
			mobj.player.sprung = true
		end
	end
end, MT_PLAYER)
SIMPLE_ANIMAL_DEFINITIONS["running_tails"] = {
	--Base animations--
  ["goal"] = {X},
--[TRICK_BEHIND] = {A-E}, -- Acrobatics plays "A-E" as the rotations for afterimages that play after you've tricked so I'm gonna leave them blank.
--[CHARACTER_SELECT] = {G}, -- G and J get used for the character select animation.  
  ["stand"] = {F},
  ["walk"] = {K, L, M, N, O, P, Q, R},
  ["run"] = {S, S, T, T},
  ["reverse"] = {R, Q, P, O, N, M, L, K},
  ["trick"] = {Y, Z, Z+1},
  ["spin"] = {V},
  ["pain"] = {U},
  ["squish"] = {Z+9},
  ["ascend"] = {Z+7},
  ["descend"] = {Z+5, Z+6},
  ["glide"] = {Z+8},  
  ["finished"] = {F},
  ["drift"] = {Z+2, Z+3, Z+4},
------------------------------------------------------------------------------------------------------------------
  ["tailswalk"] = {K, L, M, N, O, P, Q, R},  
  ["tailsrun"] = {S, S, T, T},
  ["fastrun"] = {S, T},
  ["roll"] = {Y, Z, Z+1},
  ["win"] = {H, H, H, H, H, H, H, H, H, H, H, H, H, H, H, H, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I, I},
  ["lose"] = {Z+10, Z+10, Z+10, Z+10,
			  Z+11, Z+11, Z+11, Z+11,
			  Z+12, Z+12, Z+12, Z+12,
			  Z+13, Z+13, Z+13, Z+13}
}
			--Character Specific--

function SIMPLE_ANIMAL_HELPERS.isTailsRunSpeed(mobj) -- Checks if the player is at Tails's running speed.
  local player = SIMPLE_ANIMAL_HELPERS.getPlayer(mobj)
  return player.speed > mobj.scale * 33 and player.speed <= mobj.scale * 43
end

function SIMPLE_ANIMAL_HELPERS.isTailsTopSpeed(mobj) -- Checks if the player is at Tails's top speed.
  return SIMPLE_ANIMAL_HELPERS.getPlayer(mobj).speed > mobj.scale * 43
end

function SIMPLE_ANIMAL_HELPERS.isTailsWalkSpeed(mobj) -- Checks if the player is at Tails's walking speed, but not Tails's running speed.
  local player = SIMPLE_ANIMAL_HELPERS.getPlayer(mobj)
  return player.speed > 0 and player.speed <= mobj.scale * 33
end

function SIMPLE_ANIMAL_HELPERS.isTailsWalkSpeedMultiplied(mobj, fixed_multiplier) -- Checks if the player's speed is >= Tails's walking speed multiplied (using fixed-point math.)
  local run = (mobj.scale * 33)
  return SIMPLE_ANIMAL_HELPERS.getPlayer(mobj).speed >= FixedMul(run, fixed_multiplier)
end

			--Other added helpers--

function SIMPLE_ANIMAL_HELPERS.isGliding(mobj)
  return mobj.paragliding
end

function SIMPLE_ANIMAL_HELPERS.isGrinding(mobj) -- Checks if the player is on grind rail.
  local player = SIMPLE_ANIMAL_HELPERS.getPlayer(mobj)
  return (player.grinding)
end

function SIMPLE_ANIMAL_HELPERS.isSpinDashing(mobj) -- Checks if the player is spindashing.
  local player = SIMPLE_ANIMAL_HELPERS.getPlayer(mobj)
  return player.spindash and player.spindash.active
end

function SIMPLE_ANIMAL_HELPERS.isScending(mobj) -- Checks if the player is nor rising or falling through the air.
  return mobj.momz > 0 and mobj.momz < 250000 and not P_IsObjectOnGround(mobj)
end

function SIMPLE_ANIMAL_HELPERS.isSuper(mobj) -- Checks if the player is super.
  return SIMPLE_ANIMAL_HELPERS.getPlayer(mobj).masteremerald.super
end

function SIMPLE_ANIMAL_HELPERS.isLose(mobj) -- Checks if the player is losing.
  local player = SIMPLE_ANIMAL_HELPERS.getPlayer(mobj)
  if player then
    return K_IsPlayerLosing(player)
  else
    return false
  end
end

function SIMPLE_ANIMAL_HELPERS.isWin(mobj) -- Checks if the player is winning.
  local player = SIMPLE_ANIMAL_HELPERS.getPlayer(mobj)
  if player then
    return not K_IsPlayerLosing(player)
  else
    return false
  end
end

function SIMPLE_ANIMAL_HELPERS.isSpringingDown(mobj) -- Checks if the player is falling through the air after bouncing off a spring.
  local player = SIMPLE_ANIMAL_HELPERS.getPlayer(mobj)
  return mobj.momz <= 0 and not P_IsObjectOnGround(mobj) and player.sprung == true
end

-- the regular isSpringing function is for spring pads or the pogospring item

function SIMPLE_ANIMAL_HELPERS.isSpringingUp(mobj) -- Checks if the player is falling through the air after bouncing off a spring.
  local player = SIMPLE_ANIMAL_HELPERS.getPlayer(mobj)
  return mobj.momz >= 0 and player.sprung == true
end
				--Rest of Code--


function SIMPLE_ANIMAL_HOOKS.running_tails_hook(mobj, sequences)
  if mobj.skin == "running_tails"
    local helpers = _G["SIMPLE_ANIMAL_HELPERS"]
	
	  local halfspeed = helpers.isTailsWalkSpeedMultiplied(mobj, FRACUNIT/2)
	  local quarterspeed = helpers.isTailsWalkSpeedMultiplied(mobj, FRACUNIT/4)
    if (sequences[1] == "goal")
      return
	elseif helpers.isSquished(mobj)
	  return "squish"
	elseif helpers.isPained(mobj)
	  return "pain"
	elseif helpers.isSpinning(mobj)
	  return "spin"
	elseif (helpers.isTricking(mobj) or helpers.isDropDashing(mobj) or helpers.isRespawning(mobj))
	  return "trick"
	elseif helpers.isGliding(mobj) and not helpers.isTricking(mobj)
	  return "glide"
	elseif (helpers.isGrinding(mobj) or helpers.isScending(mobj))
	  return "roll"
    elseif helpers.isAscending(mobj)
      return "ascend"
    elseif helpers.isDescending(mobj)
      return "descend"
	elseif (helpers.isCountdownCharging(mobj) or helpers.isDriftingLeft(mobj) or helpers.isDriftingRight(mobj))
      return "drift"
	elseif helpers.isTailsTopSpeed(mobj)
	  return "fastrun"
	elseif helpers.isTailsRunSpeed(mobj)
	  return "tailsrun"
    elseif helpers.isFinished(mobj) and helpers.isNoSpeed(mobj)
		if helpers.isLose(mobj)
      return "lose"
		elseif helpers.isWin(mobj)
	  return "win"
		else
	  return "stand"
		end
elseif helpers.isTailsWalkSpeed(mobj) then
    if helpers.isMovingForward(mobj) then
        if ((not quarterspeed and leveltime % 4 < 3) or (quarterspeed and not halfspeed) and leveltime % 2 < 1) then
            mobj.Simple_AnimaL_timer = mobj.Simple_AnimaL_timer - 1
        end
        return "tailswalk"
    elseif helpers.isMovingBackward(mobj) then
        if ((not quarterspeed and leveltime % 4 < 3) or (quarterspeed and not halfspeed) and leveltime % 2 < 1) then
            mobj.Simple_AnimaL_timer = mobj.Simple_AnimaL_timer - 1
        end
        return "reverse"
    end
		end
	end
end